@@ -22,11 +22,8 @@
 
 #include "nm-dns-manager.h"
 
-#include <string.h>
-
 #include "nm-dbus-interface.h"
 #include "nm-connection.h"
-
 #include "nm-client.h"
 #include "nm-object-private.h"
 #include "nm-dbus-helpers.h"
@@ -277,7 +274,7 @@ demarshal_dns_configuration (NMObject *object, GParamSpec *pspec, GVariant *valu
 		char **nameservers = NULL, **domains = NULL;
 		gboolean vpn = FALSE;
 		char *interface = NULL, *str;
-		gint priority;
+		int priority;
 
 		if (   !g_variant_lookup (entry_var, "nameservers", "as", &iterp)
 		    || !g_variant_lookup (entry_var, "priority", "i", &priority)) {
@@ -313,9 +310,9 @@ demarshal_dns_configuration (NMObject *object, GParamSpec *pspec, GVariant *valu
 		                          vpn);
 		g_free (domains);
 		g_free (nameservers);
+		g_variant_unref (entry_var);
 		if (!entry) {
 			g_warning ("Ignoring invalid DNS entry");
-			g_variant_unref (entry_var);
 			continue;
 		}
 
